;**************************************************************************
;* FileName	: I2C_SW.INC
;* Date 	: 28.02.2010
;* File version : v1.00
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Software I2C functions
;*		  ==================================
;*		  I2C_startcond
;*		  I2C_restartcond
;*		  I2C_stopcond
;*		  I2C_send
;*		  I2C_receive
;*		  I2C_ack
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines in main-modul:
;* 
;* xtal_freq	EQU	4000000	  	; crystal frequency
;* core_freq	EQU	xtal_freq/4	; core operating frequency
;*
;* #define		I2C_SCL		PORTA,0		; out
;* #define		I2C_SDA		PORTA,1		; in/out
;*
;*
;*	include "..\modules\wait.mac"
;*	include "..\modules\wait.inc"
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define DEBUG			; for some debug message
;* #define NO_I2C			; for faster simulation
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  28/02/2010 	Initial Release
;*
;**************************************************************************
	ifdef INC_I2C

;***** VARIABLES **********************************************************

		cblock
		  I2C_buffer
		  I2C_loopcounter
		endc



;***** INTERNAL MACROS *****************************************************

I2C_SDA_LO	macro
		;==========================================================
			bank1
			bcf	I2C_SDA		; tris OUT
			bank0
			bcf	I2C_SDA		; pin  LOW


		endm

I2C_SDA_HIZ	macro
		;==========================================================
			bsf	I2C_SDA		; pin  HIGH
			bank1
			bsf	I2C_SDA		; tris IN
			bank0


		endm

#define		I2C_SCL_LO	bcf	I2C_SCL
#define		I2C_SCL_HI	bsf	I2C_SCL
#define		I2C_WAIT	call	I2C_wait
#define		I2C_WAIT2	call	I2C_wait2



;***** I2C_wait ***********************************************************
;* Wait half bit
;* Input : -				; 
;* Output: -				;
;* Clock : ?		
;**************************************************************************
I2C_wait	WAIT	I2C_halftime, 2
		return


;***** I2C_wait2 **********************************************************
;* Wait quarter bit
;* Input : -				; 
;* Output: -				;
;* Clock : ?		
;**************************************************************************
I2C_wait2	WAIT	I2C_quartertime, 2
		return


;***** I2C_startcond ******************************************************
;* Generate a start/restart conditions
;* Input : -				; 
;* Output: -				;
;* Clock : ?
;**************************************************************************
I2C_startcond
I2C_restartcond	I2C_SDA_HIZ
		I2C_WAIT2
		I2C_SCL_HI
		I2C_WAIT2
		I2C_SDA_LO
		I2C_WAIT2
		I2C_SCL_LO
		I2C_WAIT2
		return




;***** I2C_stopcond *******************************************************
;* Generate a stop conditions
;* Input : -				; 
;* Output: -				;
;* Clock : ?	
;**************************************************************************
I2C_stopcond	I2C_SDA_LO
		I2C_WAIT2
		I2C_SCL_HI
		I2C_WAIT2
		I2C_SDA_HIZ
		I2C_WAIT2
		return




;***** I2C_send ***********************************************************
;* Send a byte via I2C (set Z if not acknowledged)
;* Input : w <- data			; 
;* Output: Z=1 if not acknowledged	;
;* Clock : ?	
;**************************************************************************
I2C_send	movwf	I2C_buffer
		movlw	8
		movwf	I2C_loopcounter
		goto	I2C_sendstart

I2C_sendloop	I2C_SCL_LO
		I2C_WAIT2

I2C_sendstart	btfsc	I2C_buffer,7
		goto	I2C_send_1

I2C_send_0	I2C_SDA_LO
		goto	I2C_send_go

I2C_send_1	I2C_SDA_HIZ
	
I2C_send_go	I2C_WAIT2
		I2C_SCL_HI
		I2C_WAIT

		rlf	I2C_buffer,f
		decfsz	I2C_loopcounter,f
		goto	I2C_sendloop

		I2C_SCL_LO
		I2C_SDA_HIZ

		I2C_WAIT			; read ack	
		I2C_SCL_HI			
		I2C_WAIT

		clrz
		btfsc	I2C_SDA
		setz

		I2C_SCL_LO
		I2C_WAIT2

I2C_send1	btfss	I2C_SDA			; wait until slave relases the bus
		goto	I2C_send1

		return



;***** I2C_receive ********************************************************
;* Receive a byte from I2C bus
;* Input : -				; 
;* Output: data -> w			; received data
;* Clock : ?		
;**************************************************************************
I2C_receive	clrf	I2C_buffer
		movlw	8
		movwf	I2C_loopcounter
		I2C_SDA_HIZ
		I2C_SCL_LO
	
I2C_rcvloop	I2C_WAIT2
		I2C_SCL_HI
		I2C_WAIT

		clrc
		btfsc	I2C_SDA
		setc
		rlf	I2C_buffer,f

		I2C_SCL_LO
		I2C_WAIT2

		decfsz	I2C_loopcounter,f
		goto	I2C_rcvloop

		movf	I2C_buffer,w
		return




;***** I2C_ack ************************************************************
;* Acknoledge a transfer
;* Input : C <- ack 			; 0 - no ack, 1 - ack
;* Output: -				;
;* Clock :
;**************************************************************************
I2C_ack		skipc
		goto	I2C_nack
		
		I2C_SDA_LO		; pull SDA low

I2C_nack	I2C_WAIT2
		I2C_SCL_HI		; clock pulse		
		I2C_WAIT
		I2C_SCL_LO
		I2C_WAIT2
		I2C_SDA_HIZ
		return



;**************************************************************************
	endif
